local ADDONNAME, BestScrapMeltdown = ...

-- luacheck: globals BestScrapMeltdownDB
-- luacheck: globals SLASH_BESTSCRAPMELTDOWN1 SLASH_BESTSCRAPMELTDOWN2

local _G = _G
local type, pairs, ipairs, tonumber, format, print, rawset, gmatch =
      type, pairs, ipairs, tonumber, format, print, rawset, gmatch
local match,    trim,    lower    =
      strmatch, strtrim, strlower
local GetQuestID, GetItemCount, GetNumQuestChoices, GetQuestItemLink, GetItemInfo, CreateFrame =
      GetQuestID, GetItemCount, GetNumQuestChoices, GetQuestItemLink, GetItemInfo, CreateFrame
local QuestInfoRewardsFrame, AutoCastShine_AutoCastStart, AutoCastShine_AutoCastStop =
      QuestInfoRewardsFrame, AutoCastShine_AutoCastStart, AutoCastShine_AutoCastStop

local L = setmetatable({}, {
    __index = function(t, key)
        return key
    end,
    __newindex = function(t, key, value)
        if value == true then
            rawset(t, key, key)
        end
    end,
})
do-- Localization
    local locale = GetLocale()
    L["active"] = true
L["all"] = true
L["Armour Token"] = true
L["/bsm help%s - For usage and help"] = true
L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = true
L["/bsm - Shows current Token values"] = true
L["Changed highlight mode to %q"] = true
L["help"] = true
L["mode"] = true
L["normal"] = true
L[" or "] = true
L["Usage: /bsm or /bestscrapmeltdown"] = true
L["Weapon Token"] = true
L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"


    if locale == "frFR" then
        -- L["active"] = "active"
-- L["all"] = "all"
-- L["Armour Token"] = "Armour Token"
-- L["/bsm help%s - For usage and help"] = "/bsm help%s - For usage and help"
--[==[ L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ]==]
-- L["/bsm - Shows current Token values"] = "/bsm - Shows current Token values"
-- L["Changed highlight mode to %q"] = "Changed highlight mode to %q"
-- L["help"] = "help"
-- L["mode"] = "mode"
-- L["normal"] = "normal"
-- L[" or "] = " or "
-- L["Usage: /bsm or /bestscrapmeltdown"] = "Usage: /bsm or /bestscrapmeltdown"
-- L["Weapon Token"] = "Weapon Token"
-- L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
-- L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"

    elseif locale == "deDE" then
        L["active"] = "aktiv"
L["all"] = "alle"
L["Armour Token"] = "Rüstungsmakren"
L["/bsm help%s - For usage and help"] = "/bsm help%s - Benutzung und Hilfe"
L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Ändere den hervorheben Modus.
Verfügbare Modi:
    normal%s - Benutze nur die Aufwertungsmarken
    active%s - Nutze zusätzlich die Gegenstandlevels von den Aktiven Anhänger
    all%s - Nutze alle Anhänger Gegenstandlevels]=]
L["/bsm - Shows current Token values"] = "/bsm - Zeigt den aktuellen Rüstungsmarken Wert an"
L["Changed highlight mode to %q"] = "Hervorhebungsmodus ist jetzt %q"
L["help"] = "hilfe"
L["mode"] = "modus"
L["normal"] = "normal"
L[" or "] = " oder "
L["Usage: /bsm or /bestscrapmeltdown"] = "Benutztung: /bsm or /bestscrapmeltdown"
L["Weapon Token"] = "Waffenmarken"
L["TOC/Best Scrap Meltdown"] = "Beste Schrottschmelzung"
L["TOC/Highlights the most beneficial Enhancement Token"] = "Hebt die brauchbarste Verbesserungsmarke hervor"

    elseif locale == "itIT" then
        -- L["active"] = "active"
-- L["all"] = "all"
-- L["Armour Token"] = "Armour Token"
-- L["/bsm help%s - For usage and help"] = "/bsm help%s - For usage and help"
--[==[ L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ]==]
-- L["/bsm - Shows current Token values"] = "/bsm - Shows current Token values"
-- L["Changed highlight mode to %q"] = "Changed highlight mode to %q"
-- L["help"] = "help"
-- L["mode"] = "mode"
-- L["normal"] = "normal"
-- L[" or "] = " or "
-- L["Usage: /bsm or /bestscrapmeltdown"] = "Usage: /bsm or /bestscrapmeltdown"
-- L["Weapon Token"] = "Weapon Token"
-- L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
-- L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"

    elseif locale == "koKR" then
        -- L["active"] = "active"
-- L["all"] = "all"
-- L["Armour Token"] = "Armour Token"
-- L["/bsm help%s - For usage and help"] = "/bsm help%s - For usage and help"
--[==[ L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ]==]
-- L["/bsm - Shows current Token values"] = "/bsm - Shows current Token values"
-- L["Changed highlight mode to %q"] = "Changed highlight mode to %q"
-- L["help"] = "help"
-- L["mode"] = "mode"
-- L["normal"] = "normal"
-- L[" or "] = " or "
-- L["Usage: /bsm or /bestscrapmeltdown"] = "Usage: /bsm or /bestscrapmeltdown"
-- L["Weapon Token"] = "Weapon Token"
-- L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
-- L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"

    elseif locale == "esMX" then
        -- L["active"] = "active"
-- L["all"] = "all"
-- L["Armour Token"] = "Armour Token"
-- L["/bsm help%s - For usage and help"] = "/bsm help%s - For usage and help"
--[==[ L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ]==]
-- L["/bsm - Shows current Token values"] = "/bsm - Shows current Token values"
-- L["Changed highlight mode to %q"] = "Changed highlight mode to %q"
-- L["help"] = "help"
-- L["mode"] = "mode"
-- L["normal"] = "normal"
-- L[" or "] = " or "
-- L["Usage: /bsm or /bestscrapmeltdown"] = "Usage: /bsm or /bestscrapmeltdown"
-- L["Weapon Token"] = "Weapon Token"
-- L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
-- L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"

    elseif locale == "ruRU" then
        -- L["active"] = "active"
-- L["all"] = "all"
-- L["Armour Token"] = "Armour Token"
-- L["/bsm help%s - For usage and help"] = "/bsm help%s - For usage and help"
--[==[ L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ]==]
-- L["/bsm - Shows current Token values"] = "/bsm - Shows current Token values"
-- L["Changed highlight mode to %q"] = "Changed highlight mode to %q"
-- L["help"] = "help"
-- L["mode"] = "mode"
-- L["normal"] = "normal"
-- L[" or "] = " or "
-- L["Usage: /bsm or /bestscrapmeltdown"] = "Usage: /bsm or /bestscrapmeltdown"
-- L["Weapon Token"] = "Weapon Token"
-- L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
-- L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"

    elseif locale == "zhCN" then
        -- L["active"] = "active"
-- L["all"] = "all"
-- L["Armour Token"] = "Armour Token"
-- L["/bsm help%s - For usage and help"] = "/bsm help%s - For usage and help"
--[==[ L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ]==]
-- L["/bsm - Shows current Token values"] = "/bsm - Shows current Token values"
-- L["Changed highlight mode to %q"] = "Changed highlight mode to %q"
-- L["help"] = "help"
-- L["mode"] = "mode"
-- L["normal"] = "normal"
-- L[" or "] = " or "
-- L["Usage: /bsm or /bestscrapmeltdown"] = "Usage: /bsm or /bestscrapmeltdown"
-- L["Weapon Token"] = "Weapon Token"
-- L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
-- L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"

    elseif locale == "esES" then
        -- L["active"] = "active"
-- L["all"] = "all"
-- L["Armour Token"] = "Armour Token"
-- L["/bsm help%s - For usage and help"] = "/bsm help%s - For usage and help"
--[==[ L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ]==]
-- L["/bsm - Shows current Token values"] = "/bsm - Shows current Token values"
-- L["Changed highlight mode to %q"] = "Changed highlight mode to %q"
-- L["help"] = "help"
-- L["mode"] = "mode"
-- L["normal"] = "normal"
-- L[" or "] = " or "
-- L["Usage: /bsm or /bestscrapmeltdown"] = "Usage: /bsm or /bestscrapmeltdown"
-- L["Weapon Token"] = "Weapon Token"
-- L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
-- L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"

    elseif locale == "zhTW" then
        -- L["active"] = "active"
-- L["all"] = "all"
-- L["Armour Token"] = "Armour Token"
-- L["/bsm help%s - For usage and help"] = "/bsm help%s - For usage and help"
--[==[ L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ] = [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ]==]
-- L["/bsm - Shows current Token values"] = "/bsm - Shows current Token values"
-- L["Changed highlight mode to %q"] = "Changed highlight mode to %q"
-- L["help"] = "help"
-- L["mode"] = "mode"
-- L["normal"] = "normal"
-- L[" or "] = " or "
-- L["Usage: /bsm or /bestscrapmeltdown"] = "Usage: /bsm or /bestscrapmeltdown"
-- L["Weapon Token"] = "Weapon Token"
-- L["TOC/Best Scrap Meltdown"] = "Best Scrap Meltdown"
-- L["TOC/Highlights the most beneficial Enhancement Token"] = "Highlights the most beneficial Enhancement Token"

    end
end

local QUEST_IDS = {
    [38175] = true, -- Alliance
    [38188] = true, -- Horde
}

local ARMOUR_TOKEN = 120301
local WEAPON_TOKEN = 120302

local WEAPON_ITEM_IDS = {
    [114128] = 3,
    [114129] = 6,
    [114131] = 9,
}
local ARMOUR_ITEM_IDS = {
    [114745] = 3,
    [114808] = 6,
    [114822] = 9,
}

local HIGHLIGHT_MODE_NORMAL = "NORMAL"
local HIGHLIGHT_MODE_ACTIVE_FOLLOWER = "ACTIVE_FOLLOWER"
local HIGHLIGHT_MODE_ALL_FOLLOWER = "ALL_FOLLOWER"

local HIGHLIGHT_MODES = {
    [lower(HIGHLIGHT_MODE_NORMAL)] = HIGHLIGHT_MODE_NORMAL,
    [lower(HIGHLIGHT_MODE_ACTIVE_FOLLOWER)] = HIGHLIGHT_MODE_ACTIVE_FOLLOWER,
    [lower(HIGHLIGHT_MODE_ALL_FOLLOWER)] = HIGHLIGHT_MODE_ALL_FOLLOWER,
    ["normal"] = HIGHLIGHT_MODE_NORMAL,
    ["active"] = HIGHLIGHT_MODE_ACTIVE_FOLLOWER,
    ["all"] = HIGHLIGHT_MODE_ALL_FOLLOWER,
    [lower(L["normal"])] = HIGHLIGHT_MODE_NORMAL,
    [lower(L["active"])] = HIGHLIGHT_MODE_ACTIVE_FOLLOWER,
    [lower(L["all"])] = HIGHLIGHT_MODE_ALL_FOLLOWER,
}

do-- Event Handling
    local eventFrame = CreateFrame("Frame")
    eventFrame:SetScript("OnEvent", function(frame, event, ...)
        if BestScrapMeltdown[event] then
            BestScrapMeltdown[event](BestScrapMeltdown, event, ...)
        end
    end)
    eventFrame:RegisterEvent("QUEST_COMPLETE")

    function BestScrapMeltdown:RegisterEvent(event)
        eventFrame:RegisterEvent(event)
    end
    function BestScrapMeltdown:UnregisterEvent(event)
        eventFrame:UnregisterEvent(event)
    end
end

function BestScrapMeltdown:Print(...)
    print("|cFF33FF99"..ADDONNAME.."|r:", ...)
end

do-- Chat Command
    local HELP_MODE = format(L[ [=[/bsm mode%s <mode> - Change highlight mode.
Available modes:
    normal%s - Use only Upgrade Tokens
    active%s - Add active follower equip level to the calculation
    all%s - Add all follower equip level to the calculation]=] ],
        L["mode"] ~= "mode" and L[" or "]..L["mode"] or "",
        L["normal"] ~= "normal" and L[" or "]..L["normal"] or "",
        L["active"] ~= "active" and L[" or "]..L["active"] or "",
        L["all"] ~= "all" and L[" or "]..L["all"] or ""
    )

    SLASH_BESTSCRAPMELTDOWN1, SLASH_BESTSCRAPMELTDOWN2 = "/bestscrapmeltdown", "/bsm"
    SlashCmdList.BESTSCRAPMELTDOWN = function(msg)
        msg = lower(trim(msg))

        local args = gmatch(msg, "[^%s]+")
        local arg1 = args()
        if arg1 == "mode" or arg1 == L["mode"] then
            local mode = args()

            local db_mode = HIGHLIGHT_MODES[mode]
            if db_mode then
                BestScrapMeltdownDB.highlightMode = db_mode
                BestScrapMeltdown:Print(format(L["Changed highlight mode to %q"], mode))
            else
                BestScrapMeltdown:Print(HELP_MODE)
            end
        elseif arg1 == "help" or arg1 == L["help"] then
            BestScrapMeltdown:Print(L["Usage: /bsm or /bestscrapmeltdown"])
            BestScrapMeltdown:Print(L["/bsm - Shows current Token values"])
            BestScrapMeltdown:Print(HELP_MODE)
        else
            local maxArmouriLvL, maxWeaponiLvL = BestScrapMeltdown:GetTokenValues()
            local _, itemLinkArmour = GetItemInfo(ARMOUR_TOKEN)
            local _, itemLinkWeapon = GetItemInfo(WEAPON_TOKEN)

            BestScrapMeltdown:Print(itemLinkArmour or L["Armour Token"],": ", maxArmouriLvL)
            BestScrapMeltdown:Print(itemLinkWeapon or L["Weapon Token"],": ", maxWeaponiLvL)
            BestScrapMeltdown:Print(format(L["/bsm help%s - For usage and help"], L["help"] ~= "help" and L[" or "]..L["help"] or ""))
        end
    end
end

BestScrapMeltdown:RegisterEvent("ADDON_LOADED")
function BestScrapMeltdown:ADDON_LOADED(event, addonName)
    if addonName == ADDONNAME then
        if not BestScrapMeltdownDB then
            BestScrapMeltdownDB = {}
        end

        if type(BestScrapMeltdownDB.highlightMode) ~= "string" then
            BestScrapMeltdownDB.highlightMode = HIGHLIGHT_MODE_NORMAL
        end

        self:UnregisterEvent("ADDON_LOADED")
    end
end

function BestScrapMeltdown:QUEST_FINISHED()
    self:UnregisterEvent("QUEST_FINISHED")
    self:UnregisterEvent("BAG_UPDATE")

    self:HideRewardHighlights()
end

function BestScrapMeltdown:BAG_UPDATE()
    self:QUEST_COMPLETE()
end

function BestScrapMeltdown:GetTokenValues()
    local maxArmouriLvL = 0
    local maxWeaponiLvL = 0

    for itemID, value in pairs(WEAPON_ITEM_IDS) do
        maxWeaponiLvL = maxWeaponiLvL + value * GetItemCount(itemID, true)
    end
    for itemID, value in pairs(ARMOUR_ITEM_IDS) do
        maxArmouriLvL = maxArmouriLvL + value * GetItemCount(itemID, true)
    end

    return maxArmouriLvL, maxWeaponiLvL
end

do-- GetFollowerArmourValues
    local LE_FOLLOWER_TYPE_GARRISON_6_0 = LE_FOLLOWER_TYPE_GARRISON_6_0
    local GARRISON_FOLLOWER_INACTIVE = GARRISON_FOLLOWER_INACTIVE
    local C_Garrison_GetFollowers = C_Garrison.GetFollowers
    local C_Garrison_GetFollowerItems = C_Garrison.GetFollowerItems
    local C_Garrison_GetFollowerStatus = C_Garrison.GetFollowerStatus

    local FOLLOWER_MAX_ITEM_LVL = 675

    local followers
    local missingArmouriLvLActive
    local missingArmouriLvLAll
    local missingWeaponiLvLActive
    local missingWeaponiLvLAll

    -- GARRISON_FOLLOWER_LIST_UPDATE
    function BestScrapMeltdown:GARRISON_FOLLOWER_ADDED(--[[ event, followerID, name, displayID, level, quality, isUpgraded ]])
        followers = C_Garrison_GetFollowers(LE_FOLLOWER_TYPE_GARRISON_6_0)
        --[===[@debug@
        self:Print("EVENT: GARRISON_FOLLOWER_ADDED")
        --@end-debug@]===]
    end

    function BestScrapMeltdown:GARRISON_FOLLOWER_UPGRADED(--[[event, followerID]])
        if not followers then
            self:GARRISON_FOLLOWER_ADDED()
            self:RegisterEvent("GARRISON_FOLLOWER_ADDED")
        end

        missingArmouriLvLActive = 0
        missingArmouriLvLAll = 0
        missingWeaponiLvLActive = 0
        missingWeaponiLvLAll = 0

        for i, follower in ipairs(followers) do
            if follower.isCollected and follower.level == 100 and follower.iLevel < FOLLOWER_MAX_ITEM_LVL then
                local followerID = follower.followerID
                local weaponItemID, weaponiLvl, armourItemID, armouriLvl = C_Garrison_GetFollowerItems(followerID)

                local diffArmouriLvl = FOLLOWER_MAX_ITEM_LVL - armouriLvl
                local diffWeaponiLvl = FOLLOWER_MAX_ITEM_LVL - weaponiLvl

                missingArmouriLvLAll = missingArmouriLvLAll + diffArmouriLvl
                missingWeaponiLvLAll = missingWeaponiLvLAll + diffWeaponiLvl

                if C_Garrison_GetFollowerStatus(followerID) ~= GARRISON_FOLLOWER_INACTIVE then
                    missingArmouriLvLActive = missingArmouriLvLActive + diffArmouriLvl
                    missingWeaponiLvLActive = missingWeaponiLvLActive + diffWeaponiLvl
                end
            end
        end
    end

    function BestScrapMeltdown:GetFollowerArmourValues()
        if missingArmouriLvLActive == nil or missingArmouriLvLAll == nil
        or missingWeaponiLvLActive == nil or missingWeaponiLvLAll == nil then
            self:GARRISON_FOLLOWER_UPGRADED()
            self:RegisterEvent("GARRISON_FOLLOWER_UPGRADED")
        end

        return missingArmouriLvLActive, missingWeaponiLvLActive, missingArmouriLvLAll, missingWeaponiLvLAll
    end
end

function BestScrapMeltdown:QUEST_COMPLETE()
    self:HideRewardHighlights()

    if QUEST_IDS[GetQuestID()] then
        self:RegisterEvent("QUEST_FINISHED")

        local maxArmouriLvL, maxWeaponiLvL = self:GetTokenValues()

        self:RegisterEvent("BAG_UPDATE")

        --[===[@debug@
        self:Print("maxArmouriLvL: ", maxArmouriLvL)
        self:Print("maxWeaponiLvL: ", maxWeaponiLvL)
        --@end-debug@]===]

        if BestScrapMeltdownDB.highlightMode ~= HIGHLIGHT_MODE_NORMAL then
            local missingArmouriLvLActive, missingWeaponiLvLActive, missingArmouriLvLAll, missingWeaponiLvLAll = self:GetFollowerArmourValues()

            --[===[@debug@
            self:Print("HighlightMode:", BestScrapMeltdownDB.highlightMode)
            --@end-debug@]===]

            if BestScrapMeltdownDB.highlightMode == HIGHLIGHT_MODE_ACTIVE_FOLLOWER then
                maxArmouriLvL = maxArmouriLvL - missingArmouriLvLActive
                maxWeaponiLvL = maxWeaponiLvL - missingWeaponiLvLActive
            elseif BestScrapMeltdownDB.highlightMode == HIGHLIGHT_MODE_ALL_FOLLOWER then
                maxArmouriLvL = maxArmouriLvL - missingArmouriLvLAll
                maxWeaponiLvL = maxWeaponiLvL - missingWeaponiLvLAll
            end
        end

        --[===[@debug@
        self:Print("maxArmouriLvL: ", maxArmouriLvL)
        self:Print("maxWeaponiLvL: ", maxWeaponiLvL)
        --@end-debug@]===]

        if maxArmouriLvL < maxWeaponiLvL then
            for i = 1, GetNumQuestChoices() do
                local link = GetQuestItemLink("choice", i)
                local itemID = tonumber( match(link, "item:(%d+)") )
                if itemID == ARMOUR_TOKEN then
                    self:SetRewardHighlight(i)
                    break
                end
            end
        elseif maxWeaponiLvL < maxArmouriLvL then
            for i = 1, GetNumQuestChoices() do
                local link = GetQuestItemLink("choice", i)
                local itemID = tonumber( match(link, "item:(%d+)") )
                if itemID == WEAPON_TOKEN then
                    self:SetRewardHighlight(i)
                    break
                end
            end
        else
            for i = 1, GetNumQuestChoices() do
                local link = GetQuestItemLink("choice", i)
                local itemID = tonumber( match(link, "item:(%d+)") )

                if itemID == ARMOUR_TOKEN then
                    self:SetRewardHighlight(i)
                end
                if itemID == WEAPON_TOKEN then
                    self:SetRewardHighlight(i)
                end
            end
        end
    end
end

do-- Reward Highlight
    local rewardHighlightFrames = {}
    local function rewardHighlightFrame_OnHide(self)
        AutoCastShine_AutoCastStop(self)
    end

    -- Original code: Quester from Nevcairiel
    function BestScrapMeltdown:SetRewardHighlight(rewardID)
        local frame = rewardHighlightFrames[rewardID]

        if not frame then
            frame = CreateFrame("Frame", "BestScrapMeltdownRewardHighlight"..rewardID, QuestInfoRewardsFrame, "AutoCastShineTemplate")
            frame:SetScript("OnHide", rewardHighlightFrame_OnHide)
            rewardHighlightFrames[rewardID] = frame

            frame:ClearAllPoints()
            frame:SetAllPoints( _G[ format("QuestInfoRewardsFrameQuestInfoItem%dIconTexture", rewardID) ] )
        end

        frame:Show()
        AutoCastShine_AutoCastStart(frame)
    end

    function BestScrapMeltdown:HideRewardHighlights()
        for _, frame in pairs(rewardHighlightFrames) do
            frame:Hide()
        end
    end
end

